from flask import render_template, abort, redirect, url_for, abort, flash, request, current_app, make_response, g
from flask_login import login_required, login_user, logout_user, current_user
from flask_sqlalchemy import get_debug_queries

from . import panel
from .forms import OrderForm
from .. import db
from ..email import send_email
from ..models import User, Permission, Server, Deployment, Service, Region, Address, Domain, contact_proxmaster

import base64

@panel.after_app_request
def after_request(response):
    for query in get_debug_queries():
        if query.duration >= current_app.config['SLOW_DB_QUERY_TIME']:
            current_app.logger.warning('Slow query: %s\nParameters: %s\nDuration: %fs\nContext: %s\n' % (query.statement, query.parameters, query.duration, query.context))
    return response

@panel.route("/order", methods=['GET', 'POST'])
def order():
    form = OrderForm()
    if form.validate_on_submit():
        return redirect('main.index')
    return render_template('panel/order.html', form=form)

#DASHBOARD
@panel.route("/dashboard", methods=['GET', 'POST'])
@login_required
def dashboard():
    sys_regions = Region.query.all()

    cuser = current_user
    inv_addresses = cuser.inv_addresses.order_by(Address.ip.asc()).all()
    inv_deployments = cuser.inv_deployments.filter_by(deleted=False).order_by(Deployment.date_created.desc()).all()
    regions = {}
    for region in sys_regions:
        regions[region.pid] = region.description

    inv_deploycubeids = []
    for invcls in inv_deployments: 
        if invcls.user_id == cuser.pid:
            inv_deploycubeids.extend([invcls.machine_id]) 

    inv_services = cuser.inv_services.filter_by(deleted=False).order_by(Service.date_last_charge.asc()).all()
    inv_domains = cuser.inv_domains.filter_by(deleted=False).order_by(Domain.date_created.desc()).all()

    #extract rrd and status from the deployments
    rrd = {}
    statuses = {}
    #current_app.logger.warning(str(inv_deploycubeids))
    for unit_id in inv_deploycubeids:
        rrd[unit_id] = {}
        data = { 'unit_id': int(unit_id),
                 'type': 'kvm' }
        try:
            query = contact_proxmaster(data, 'vmrrd')
        except Exception as e:
            current_app.logger.error(e)
            flash('Support is notified.'.format(str(unit_id)))
            send_email(current_app.config['MAIL_USERNAME'], 'Cube {} is unreachable'.format(unit_id), 
                       'vmanager/email/adm_unreachable', user=current_user, unit_id=unit_id, error=str(e))
        #current_app.logger.info('debug query:')
        #current_app.logger.info(query)

        graphs_list = ['net', 'cpu', 'mem', 'hdd']
        try: 
            for graph in graphs_list:
                raw = query[graph]['image'].encode('raw_unicode_escape')
                rrd[unit_id][graph] = base64.b64encode(raw).decode()
            status = { unit_id : query['status'] }
            statuses.update(status)
        except Exception as e:
            current_app.logger.error(e)
            flash('Support is notified.'.format(str(unit_id)))
            send_email(current_app.config['MAIL_USERNAME'], 'Cube {} is unreachable'.format(unit_id), 
                                          'vmanager/email/adm_unreachable', user=current_user, unit_id=unit_id, error=str(e))
    
    return render_template('panel/dashboard.html', sys_regions=sys_regions, inv_deployments=inv_deployments, inv_services=inv_services, inv_domains=inv_domains, inv_addresses=inv_addresses, rrd=rrd, status=statuses, regions=regions)

